"""
ZSHPower
~~~~~~~~

ZSHPower is a theme for ZSH with a manager.


For more information, access: 'https://github.com/snakypy/zshpower'

:copyright: Copyright 2020-2021 by Snakypy team, see AUTHORS.
:license: MIT license, see LICENSE for details.
"""

from contextlib import suppress
from os.path import abspath, dirname, join
from pathlib import Path

from snakypy.helpers.files import eqversion

with suppress(KeyboardInterrupt):
    HOME = str(Path.home())
    __info__ = {
        "name": "ZSHPower",
        "version": "0.11.7",
        "description": "ZSHPower is a theme for ZSH with a manager.",
        "pkg_name": "zshpower",
        "executable": "zshpower",
        "home_page": "https://github.com/snakypy/zshpower",
        "organization_name": "Snakypy",
        "author": {
            "name": "William C. Canin",
            "email": "william.costa.canin@gmail.com",
            "website": "https://williamcanin.github.io",
            "github": "https://github.com/williamcanin",
        },
        "credence": [
            {
                "my_name": "William C. Canin",
                "email": "william.costa.canin@gmail.com",
                "website": "https://williamcanin.github.io",
                "locale": "Brazil - SP",
            }
        ],
    }

# Keep the versions the same on pyproject.toml and __init__.py
pyproject = join(dirname(abspath(__file__)), "../..", "pyproject.toml")
eqversion(pyproject, __info__["version"])



from datetime import datetime

config_content: str = f"""# Generate by: ZSHPower - D{datetime.today().isoformat()}
# ---------------------------------------------------------------------
# For more information, see the documentation at:
# https://github.com/snakypy/zshpower#configuration-file

[general]
color.enable = true
jump_line.enable = true
config.editor = "vim"
separator.element = "∙"
separator.color = "negative"
position = [
    "virtualenv",
    "python",
    "package",
    "nodejs",
    "c",
    "cpp",
    "rust",
    "golang",
    "java",
    "php",
    "ruby",
    "elixir",
    "ember",
    "julia",
    "dart",
    "dotnet",
    "docker",
    "perl",
    "scala",
    "cmake",
    "crystal",
    "deno",
    "erlang",
    "helm",
    "kotlin",
    "lua",
    "nim",
    "ocaml",
    "vagrant",
    "v",
    "zig",
    "gulp",
    "git",
    ]

[command]
new_line.enable = true
symbol = "\\u276f"
color = "green"
error.symbol = "\\u276f"
error.color = "red"

[c]
symbol = "\\ue61e"
color = "red"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[cpp]
symbol = "\\ue61d"
color = "red"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[cmake]
symbol = "\\ufa35"
color = "green"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[crystal]
symbol = "\\uf7d7"
color = "magenta"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[dart]
symbol = "\\ue798"
color = "cyan"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[deno]
symbol = "\\u1f995"
color = "green"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[directory]
truncation_length = 1
symbol = "\\ue5fe"
color = "cyan"
prefix.color = "negative"
prefix.text = "in"

[docker]
symbol = "\\uf308"
color = "cyan"
prefix.color = "negative"
prefix.text = "on"
version.enable = false
version.micro.enable = true

[dotnet]
symbol = "\\ue77f"
color = "cyan"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[elixir]
symbol = "\\ue62d"
color = "blue"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[ember]
symbol = "\\ue71b"
color = "red"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = false

[erlang]
symbol = "\\ue7b1"
color = "red"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[git]
enable = true
symbol = "\\ue725"
branch.color = "cyan"
color.symbol = "magenta"
prefix.color = "negative"
prefix.text = "on"

[git.status]
symbols.enable = true
symbol.clean = "\\uf62c"
symbol.added = "\\ufc03"
symbol.modified = "\\ufc07"
symbol.deleted = "\\ufbc7"
symbol.renamed = "\\uf101"
symbol.unmerged = "\\uf6fb"
symbol.untracked = "\\uf41e"
symbol.copied = "\\ufab1"
symbol.ahead = "\\uf55c"
symbol.behind = "\\uf544"
symbol.diverged = "\\ufb15"
symbol.conflicts = "\\uf0e7"

[golang]
symbol = "\\ue627"
color = "cyan"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[gulp]
symbol = "\\ue763"
color = "red"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[helm]
symbol = "\\ufd31"
color = "cyan"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[hostname]
enable = false
symbol = "\\ue0a2"
color = "magenta"
prefix.color = "negative"
prefix.text = "at"

[java]
symbol = "\\ue256"
color = "red"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[julia]
symbol = "\\ue624"
color = "blue"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[kotlin]
symbol = "\\ue622"
color = "blue"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[lua]
symbol = "\\ue620"
color = "blue"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[nim]
symbol = "\\uf6a4"
color = "yellow"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[nodejs]
symbol = "\\uf898"
color = "green"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[ocaml]
symbol = "ocaml"
color = "yellow"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[package]
enable = false
symbol = "\\uf8d6"
display = [
    "python",
    "ruby",
    "node",
    "rust",
    "scala",
    "crystal",
    "helm",
    "v"
]
color = "red"
prefix.color = "negative"
prefix.text = "is"

[perl]
symbol = "\\ue769"
color = "blue"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[php]
symbol = "\\ue608"
color = "magenta"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[python]
symbol = "\\uf81f"
color = "yellow"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[python.virtualenv]
enable = false
symbol = "\\uf7c9"
involved = "[]"
color = "yellow"
prefix.color = "negative"
prefix.text = "via"

[python.virtualenv.name]
normal.enable = true
text = "venv"

[python.virtualenv.poetry]
py.enable = true
hash.enable = true

[ruby]
symbol = "\\ue21e"
color = "red"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[rust]
symbol = "\\ue7a8"
color = "red"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[r]
symbol = "R"
color = "blue"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[scala]
symbol = "\\ue737"
color = "red"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[timer]
enable = false
symbol = "\\uf43a"
color = "blue"
seconds.enable = false

[took]
enable = false
symbol = "\\ufbab"
text = "took"
color = "yellow"
involved = "[]"
show_greater_than = 2.5

[username]
enable = false
symbol = "\\uf007"
color = "cyan"

[vagrant]
symbol = "\\ue62b"
color = "yellow"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[v]
symbol = "V"
color = "blue"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true

[zig]
symbol = "\\ue00a"
color = "yellow"
prefix.color = "negative"
prefix.text = "via"
version.enable = false
version.micro.enable = true
"""



from snakypy.helpers import pick, printer
from snakypy.helpers.ansi import FG
from snakypy.helpers.decorators import silent_errors
from snakypy.helpers.files import backup_file, create_file
from snakypy.helpers.os import remove_objects

from snakypy.zshpower import HOME, __info__
from snakypy.zshpower.commands.cron import Cron
from snakypy.zshpower.config.base import Base
from snakypy.zshpower.config.zshrc import zshrc_sample
from snakypy.zshpower.utils.catch import get_zsh_theme
from snakypy.zshpower.utils.modifiers import change_theme, pip_uninstall, remove_lines
from snakypy.zshpower.utils.process import reload_zsh


class UninstallCommand(Base):
    def __init__(self, home: str):
        Base.__init__(self, home)

    def using_omz(self) -> bool:
        check = get_zsh_theme(self.zsh_rc, self.logfile)
        if check:
            return True
        return False

    def zshpower_with_omz(self) -> str:
        title = "What did you want to uninstall?"
        options = [
            f"{__info__['name']}",
            f"{__info__['name']} and Oh My ZSH",
            "Cancel",
        ]
        reply = pick(title, options, colorful=True, index=True)
        # Cancel
        if reply is None or reply[0] == 2:
            printer("Whew! Thanks! :)", foreground=FG().GREEN)
            exit(0)
        # Remove structure ZSHPower
        # Note: Add folders first then files.
        remove_objects(
            objects=(
                self.database_root,
                self.cache_root,
                self.lib_root,
                self.theme_symlink,
            )
        )
        Cron(HOME).manager(action="remove")
        pip_uninstall(packages=(__info__["name"],))
        remove_lines(
            self.zsh_rc,
            self.logfile,
            lines=('eval "\\$\\(zshpower init --path\\)"',),
        )
        change_theme(self.zsh_rc, "robbyrussell", self.logfile)
        # ZSHPower and Oh My ZSH
        if reply[0] == 1:
            backup_file(self.zsh_rc, self.zsh_rc, date=True, extension=False)
            remove_objects(
                objects=(
                    self.omz_root,
                    self.zsh_rc,
                )
            )
            create_file(zshrc_sample, self.zsh_rc, force=True)
        printer("Uninstallation completed!", foreground=FG().FINISH)
        reload_zsh(sleep_timer=2, message=True)
        return "ZSHPower removed!"

    def orphan_zshpower(self):
        title = f"Do you want to uninstall {__info__['name']}?"
        options = ["Yes", "No"]
        reply = pick(title, options, colorful=True, index=True)
        if reply is None or reply[0] == 1:
            printer("Whew! Thanks! :)", foreground=FG().GREEN)
            exit(0)
        remove_objects(
            objects=(
                self.database_root,
                self.cache_root,
                self.lib_root,
            )
        )
        Cron(HOME).manager(action="remove")
        pip_uninstall(packages=(__info__["name"],))
        remove_lines(
            self.zsh_rc,
            self.logfile,
            lines=(
                '\\[\\[ -d "\\$HOME/.zshpower/lib" \\]\\] && eval "\\$\\(zshpower init --path\\)"',
                'eval "\\$\\(zshpower init --path\\)"',
            ),
        )
        reload_zsh(sleep_timer=2, message=True)
        return "ZSHPower removed!"

    @silent_errors
    def run(self) -> str:
        if self.using_omz():
            return self.zshpower_with_omz()
        return self.orphan_zshpower()



from snakypy.helpers import printer
from snakypy.helpers.ansi import FG
from snakypy.helpers.catches.generic import whoami

from snakypy.zshpower.config.base import Base
from snakypy.zshpower.utils.catch import get_zsh_theme
from snakypy.zshpower.utils.check import checking_init
from snakypy.zshpower.utils.modifiers import change_theme
from snakypy.zshpower.utils.process import reload_zsh


class ActivateCommand(Base):
    def __init__(self, home: str):
        Base.__init__(self, home)

    def run(self) -> bool:

        checking_init(self.HOME, self.logfile)

        if get_zsh_theme(self.zsh_rc, self.logfile):
            if get_zsh_theme(self.zsh_rc, self.logfile)[0] == "zshpower":
                printer("Already activated. Nothing to do.", foreground=FG().GREEN)
                exit(0)
            change_theme(self.zsh_rc, "zshpower", self.logfile)
            printer("Activation process finish.", foreground=FG().FINISH)
            self.log.record(
                f"User ({whoami()}) has enabled ZSHPower.", colorize=True, level="info"
            )
            reload_zsh()
            return True
        printer(
            "You are not using Oh My ZSH to run this command.",
            foreground=FG().WARNING,
        )
        return False



from snakypy.helpers.catches import whoami

from snakypy.zshpower import HOME

cron_content: str = f"""# ZSHPower Tasks  - BEGIN
SHELL=/bin/zsh
PATH=\"\\$PATH:/bin:/usr/local/bin:{HOME}/.local/bin\"
export PATH


## ZSHPower Tasks (Every 2 minutes of every hour) ##
# */2 * * * * {whoami()} zshpower sync
# */2 * * * * {whoami()} zshpower logs --clean

## ZSHPower Tasks (Every 5 minutes of every hour) ##
# */5 * * * * {whoami()} zshpower sync
# */5 * * * * {whoami()} zshpower logs --clean

## ZSHPower Tasks (Every 10 minutes of every hour) ##
# */10 * * * * {whoami()} zshpower sync
# */10 * * * * {whoami()} zshpower logs --clean

## ZSHPower Tasks (Every 30 minutes of every hour) ##
# */30 * * * * {whoami()} zshpower sync
# */30 * * * * {whoami()} zshpower logs --clean

## ZSHPower Tasks (At minute 0 past every 2nd hour) ##
# 0 */2 * * * {whoami()} zshpower sync
# 0 */2 * * * {whoami()} zshpower logs --clean

## ZSHPower Tasks (Daily at 7:00 am) ##
# 0 07 * * * {whoami()} zshpower sync
# 0 07 * * * {whoami()} zshpower logs --clean

# ZSHPower Tasks (Monthly)
# 0 0 1 * * {whoami()} zshpower sync
# 0 0 1 * * {whoami()} zshpower logs --clean

# ZSHPower Tasks (Weekly)
# 0 0 * * 1 {whoami()} zshpower sync
# 0 0 * * 1 {whoami()} zshpower logs --clean

# ZSHPower Tasks (Daily)
# 0 0 * * * {whoami()} zshpower sync
# 0 0 * * * {whoami()} zshpower logs --clean

# You can use the site "https://crontab.guru" and some examples at "https://crontab.guru/examples.html"
#
# ZSHPower Tasks - END"""

# sync_content: str = f"""
# #!/usr/bin/env zsh
#
# SHELL=/bin/zsh
# PATH=\"\\$PATH:/bin:/usr/local/bin:{HOME}/.local/bin\"
# export PATH
#
# if [[ -f $(which zshpower) ]]; then
#   zshpower sync
# else
#     echo "Not found command 'zshpower'."
# fi
#
# """



from subprocess import run

from snakypy.zshpower.config.base import Base
from snakypy.zshpower.prompt.sections.utils import Version


class Zig(Version, Base):
    def __init__(self, *args):
        super(Zig, self).__init__()
        self.args: tuple = args
        self.key = "zig"
        self.app_executable = "zig"
        self.shorten = "zig-"
        self.finder = {"extensions": [".zig"], "folders": [], "files": []}

    def get_version(self, space_elem: str = " ") -> str:
        # args[0]: dict = config file (toml)
        # args[1]: dict = database registers
        return super().get(
            self.args[0], self.args[1], self.key, self.shorten, space_elem=space_elem
        )

    def set_version(self, action: str = "") -> bool:
        command = run(
            f"{self.app_executable} version", capture_output=True, shell=True, text=True
        )
        version = command.stdout.replace("\n", "")
        return super().set(
            command, version, self.app_executable, self.key, action=action
        )

    def __str__(self):
        return self.get_version()



from os import environ, getcwd
from os.path import exists, isdir, isfile, join
from sys import version_info

from snakypy.helpers.catches import is_tool
from snakypy.helpers.files import read_file

from snakypy.zshpower import HOME
from snakypy.zshpower.prompt.sections.utils import (
    Color,
    element_spacing,
    separator,
    symbol_ssh,
)
from snakypy.zshpower.utils.catch import get_key, verify_objects


def definitive_version(
    micro_version_enable: bool, python_version: list, space_elem: str
):
    if not micro_version_enable:
        return f"{'{0[0]}.{0[1]}'.format(python_version)}{space_elem}"
    return f"{'{0[0]}.{0[1]}.{0[2]}'.format(python_version)}{space_elem}"


class Python:
    def __init__(self, config: dict, *args):

        self.config: dict = config
        self.args: tuple = args
        self.enable: bool = get_key(config, "python", "version", "enable")
        self.finder = {
            "extensions": [".py"],
            "folders": ["__pycache__"],
            "files": [
                "manage.py",
                "setup.py",
                "__init__.py",
                ".python-version",
                "requirements.txt",
                "pyproject.toml",
            ],
        }

        self.symbol = symbol_ssh(get_key(config, "python", "symbol"), "py-")
        self.color = (
            get_key(config, "python", "color")
            if get_key(config, "general", "color", "enable") is True
            else "negative"
        )
        self.prefix_color = get_key(config, "python", "prefix", "color")
        self.prefix_text = element_spacing(get_key(config, "python", "prefix", "text"))
        self.micro_version_enable: bool = get_key(
            config, "python", "version", "micro", "enable"
        )

    def get_version(self, space_elem: str = " ") -> str:

        # Checking if you use Python through pyenv or the system.
        if isdir(join(HOME, ".pyenv")):

            pyenv_file_local = join(getcwd(), self.finder["files"][3])
            pyenv_file_global = join(HOME, ".pyenv/version")

            if exists(pyenv_file_local):
                if read_file(pyenv_file_local).strip() == "system":
                    python_version = (
                        f"{'{0[0]}.{0[1]}.{0[2]}'.format(version_info)}".split(".")
                    )
                else:
                    python_version = read_file(pyenv_file_local).strip().split(".")
            elif exists(pyenv_file_global):
                if read_file(pyenv_file_global).strip() == "system":
                    python_version = (
                        f"{'{0[0]}.{0[1]}.{0[2]}'.format(version_info)}".split(".")
                    )
                else:
                    python_version = read_file(pyenv_file_global).strip().split(".")
            else:
                python_version = f"{'{0[0]}.{0[1]}.{0[2]}'.format(version_info)}".split(
                    "."
                )
        else:
            python_version = f"{'{0[0]}.{0[1]}.{0[2]}'.format(version_info)}".split(".")

        return definitive_version(self.micro_version_enable, python_version, space_elem)

    def __str__(self):
        if self.enable:
            if is_tool("python") or is_tool("python3"):
                if (
                    verify_objects(getcwd(), data=self.finder) is True
                    or "VIRTUAL_ENV" in environ
                ):
                    prefix = (
                        f"{Color(self.prefix_color)}{self.prefix_text}{Color().NONE}"
                    )

                    return str(
                        f"{separator(self.config)}{prefix}"
                        f"{Color(self.color)}{self.symbol}"
                        f"{self.get_version()}{Color().NONE}"
                    )
        return ""


class Virtualenv:
    def __init__(self, config: dict, *args):
        self.config: dict = config
        self.args = args
        self.enable: bool = get_key(config, "python", "virtualenv", "enable")
        self.hash_enable: bool = get_key(
            config, "python", "virtualenv", "poetry", "hash", "enable"
        )
        self.py_enable = get_key(
            config, "python", "virtualenv", "poetry", "py", "enable"
        )
        self.symbol = symbol_ssh(get_key(config, "python", "virtualenv", "symbol"), "")
        self.involved = get_key(config, "python", "virtualenv", "involved")
        self.color = (
            get_key(config, "python", "virtualenv", "color")
            if get_key(config, "general", "color", "enable") is True
            else "negative"
        )
        self.prefix_color = get_key(config, "python", "virtualenv", "prefix", "color")
        self.prefix_text = element_spacing(
            get_key(config, "python", "virtualenv", "prefix", "text")
        )
        self.name_enable = get_key(
            config, "python", "virtualenv", "name", "normal", "enable"
        )
        self.name_text = get_key(config, "python", "virtualenv", "name", "text")

    def get_virtualenv(self) -> str:
        if "VIRTUAL_ENV" in environ:
            venv_path = environ["VIRTUAL_ENV"]
            # The "venv_hash" option is only for virtual machines created with
            # poetry, where it generates a hash in the name of the virtual machine.
            if not self.hash_enable and isfile(join(getcwd(), "pyproject.toml")):
                venv_path = environ["VIRTUAL_ENV"].split("/")[-1]
                if "-" in venv_path:
                    return "-".join(venv_path.split("-")[:-2])
            # The "pg_version" option is only for virtual machines created with
            # poetry, where it generates the current version of Python in the name
            # of the virtual machine.
            if not self.py_enable and isfile(join(getcwd(), "pyproject.toml")):
                if "-" in venv_path:
                    return "-".join(venv_path.split("/")[-1].split("-")[:-1])

            return venv_path.split("/")[-1]
        return ""

    def __str__(self, space_elem: str = " "):
        involved_prefix = ""
        involved_suffix = ""

        if self.enable:
            if "VIRTUAL_ENV" in environ:
                prefix = (
                    f"{Color(self.prefix_color)}" f"{self.prefix_text}{Color().NONE}"
                )
                if len(self.involved) == 2:
                    involved_prefix = self.involved[0]
                    involved_suffix = self.involved[1]

                if self.name_enable:
                    ret = (
                        f"{separator(self.config)}{prefix}{Color(self.color)}"
                        f"{self.symbol}"
                        f"{involved_prefix}{self.get_virtualenv()}{involved_suffix}"
                        f"{space_elem}{Color().NONE}"
                    )
                else:
                    ret = (
                        f"{separator(self.config)}{prefix}{Color(self.color)}"
                        f"{self.symbol}"
                        f"{involved_prefix}{self.name_text}{involved_suffix}"
                        f"{space_elem}{Color().NONE}"
                    )
                return str(ret)
        return ""



from os import environ, getcwd
from os.path import isdir, join

from snakypy.zshpower.prompt.sections.utils import (
    Color,
    element_spacing,
    git_status,
    separator,
    symbol_ssh,
)
from snakypy.zshpower.utils.catch import get_key


class Git:
    def __init__(self, config: dict, *args):

        self.config: dict = config
        self.database = args[0]
        self.icon_space: str = " "
        try:
            self.symbol = symbol_ssh(get_key(config, "git", "symbol"), "git:")
        except KeyError:
            self.symbol = symbol_ssh("\uf418", "git:")
        self.gcolor_enable = get_key(config, "general", "color", "enable")
        self.enable = get_key(config, "git", "enable")
        self.color_symbol = (
            get_key(config, "git", "color", "symbol")
            if self.gcolor_enable is True
            else "negative"
        )
        self.branch_color = (
            get_key(config, "git", "branch", "color")
            if self.gcolor_enable is True
            else "negative"
        )
        self.prefix_color = (
            get_key(config, "git", "prefix", "color")
            if self.gcolor_enable is True
            else "negative"
        )
        self.prefix_text = element_spacing(get_key(config, "git", "prefix", "text"))
        self.symbol_enable = get_key(config, "git", "status", "symbols", "enable")
        self.icons = {
            "A": [
                f"{Color('green') if self.gcolor_enable is True else Color('negative')}"
                f"{symbol_ssh(get_key(config, 'git', 'status', 'symbol', 'added'), '')}"
                f"{Color().NONE}",
                f"{Color('green') if self.gcolor_enable is True else Color('negative')}+"
                f"{self.icon_space}{Color().NONE}",
            ],
            # "AM": [
            #     f"{Color('white') if get_key(config, 'general', 'color', 'enable') is True else Color('negative')}"
            #     f"{symbol_ssh(get_key(config, 'git', 'status', 'symbol', 'changed'), '')}"
            #     f"{Color().NONE}",
            #     f"{Color('white') if self.gcolor_enable is True else Color('negative')}#
            #     {self.icon_space}{Color().NONE}",
            # ],
            "M": [
                f"{Color('blue') if self.gcolor_enable is True else Color('negative')}"
                f"{symbol_ssh(get_key(config, 'git', 'status', 'symbol', 'modified'), '')}"
                f"{Color().NONE}",
                f"{Color('blue') if self.gcolor_enable is True else Color('negative')}#"
                f"{self.icon_space}{Color().NONE}",
            ],
            "D": [
                f"{Color('red') if self.gcolor_enable is True else Color('negative')}"
                f"{symbol_ssh(get_key(config, 'git', 'status', 'symbol', 'deleted'), '')}"
                f"{Color().NONE}",
                f"{Color('red') if self.gcolor_enable is True else Color('negative')}x"
                f"{self.icon_space}{Color().NONE}",
            ],
            "??": [
                f"{Color('yellow') if self.gcolor_enable is True else Color('negative')}"
                f"{symbol_ssh(get_key(config, 'git', 'status', 'symbol', 'untracked'), '')}"
                f"{Color().NONE}",
                f"{Color('yellow') if self.gcolor_enable is True else Color('negative')}?"
                f"{self.icon_space}{Color().NONE}",
            ],
            "R": [
                f"{Color('magenta') if self.gcolor_enable is True else Color('negative')}"
                f"{symbol_ssh(get_key(config, 'git', 'status', 'symbol', 'renamed'), '')}"
                f"{Color().NONE}",
                f"{Color('magenta') if self.gcolor_enable is True else Color('negative')}->"
                f"{self.icon_space}{Color().NONE}",
            ],
            "UU": [
                f"{Color('red') if self.gcolor_enable is True else Color('negative')}"
                f"{symbol_ssh(get_key(config, 'git', 'status', 'symbol', 'conflicts'), '')}"
                f"{Color().NONE}",
                f"{Color('red') if self.gcolor_enable is True else Color('negative')}!="
                f"{self.icon_space}{Color().NONE}",
            ],
            "AH": [
                f"{Color('blue') if self.gcolor_enable is True else Color('negative')}"
                f"{symbol_ssh(get_key(config, 'git', 'status', 'symbol', 'ahead'), '')}"
                f"{Color().NONE}",
                f"{Color('blue') if self.gcolor_enable is True else Color('negative')}^"
                f"{self.icon_space}{Color().NONE}",
            ],
            "BH": [
                f"{Color('magenta') if self.gcolor_enable is True else Color('negative')}"
                f"{symbol_ssh(get_key(config, 'git', 'status', 'symbol', 'behind'), '')}"
                f"{Color().NONE}",
                f"{Color('magenta') if self.gcolor_enable is True else Color('negative')}_"
                f"{self.icon_space}{Color().NONE}",
            ],
            "DG": [
                f"{Color('yellow') if self.gcolor_enable is True else Color('negative')}"
                f"{symbol_ssh(get_key(config, 'git', 'status', 'symbol', 'diverged'), '')}"
                f"{Color().NONE}",
                f"{Color('yellow') if self.gcolor_enable is True else Color('negative')}<->"
                f"{self.icon_space}{Color().NONE}",
            ],
            "C": [
                f"{Color('yellow') if self.gcolor_enable is True else Color('negative')}"
                f"{symbol_ssh(get_key(config, 'git', 'status', 'symbol', 'copied'), '')}"
                f"{Color().NONE}",
                f"{Color('yellow') if self.gcolor_enable is True else Color('negative')}**"
                f"{self.icon_space}{Color().NONE}",
            ],
            "U": [
                f"{Color('magenta') if self.gcolor_enable is True else Color('negative')}"
                f"{symbol_ssh(get_key(config, 'git', 'status', 'symbol', 'unmerged'), '')}"
                f"{Color().NONE}",
                f"{Color('magenta') if self.gcolor_enable is True else Color('negative')}="
                f"{self.icon_space}{Color().NONE}",
            ],
            "CL": [
                f"{Color('green') if self.gcolor_enable is True else Color('negative')}"
                f"{symbol_ssh(get_key(config, 'git', 'status', 'symbol', 'clean'), '')}"
                f"{Color().NONE}",
                f"{Color('green') if self.gcolor_enable is True else Color('negative')}~"
                f"{self.icon_space}{Color().NONE}",
            ],
        }
        self.icons["UD"] = self.icons["UU"]

    def __str__(self):
        if isdir(join(getcwd(), ".git")) and self.enable:
            status_git = git_status(porcelain=True)
            status_git_text = git_status()
            branch_current = git_status(branch=True)
            branch_formated = (
                f"{Color(self.prefix_color)}"
                f"{self.prefix_text}{Color().NONE}"
                f"{Color(self.color_symbol)}"
                f"{self.symbol}{Color().NONE}"
                f"{Color(self.branch_color)}"
                f"{branch_current}"
                f"{Color().NONE}"
            )
            status_current = []
            if "??" in status_git:
                status_current.append("??")
            if "D" in status_git:
                status_current.append("D")
            if "R" in status_git:
                status_current.append("R")
            if "A" in status_git:
                status_current.append("A")
            # if "AM" in status_git:
            #     status_current.append("AM")
            if "M" in status_git:
                status_current.append("M")
            if "UU" in status_git:
                status_current.append("UU")
            if "U" in status_git:
                status_current.append("U")
            if "UD" in status_git:
                status_current.append("UD")
            if "C" in status_git:
                status_current.append("C")
            if "ahead" in status_git_text:
                status_current.append("AH")
            if "behind" in status_git_text:
                status_current.append("BH")
            if "diverged" in status_git_text:
                status_current.append("DG")
            if len(status_git) == 0:
                status_current.append("CL")

            status_icons = (
                self.icons[item][0]
                if self.symbol_enable
                else self.icons[item][1]
                if "SSH_CONNECTION" not in environ
                else self.icons[item][1]
                for item in status_current
            )

            # status = f'( {" ".join(sorted(status_icons)).strip()} )'
            # if len(status_current) == 1:
            status = "".join(sorted(status_icons))

            return f"{separator(self.config)}{branch_formated} {status}"
        return ""



import os
from shutil import which
from subprocess import call
from time import sleep

from snakypy.helpers import FG, printer
from snakypy.helpers.catches import is_tool, shell, whoami
from snakypy.helpers.files import read_file
from snakypy.helpers.logging import Log
from tomlkit import parse as toml_parse

from snakypy.zshpower.utils.catch import get_key


def reload_zsh(sleep_timer: int = 0, message: bool = False) -> None:
    """
    Reload ZSH
    """
    if message:
        printer("Restarting terminal, wait...", foreground=FG().QUESTION)
    if sleep_timer:
        sleep(sleep_timer)
    call("exec zsh", shell=True)


def change_shell(logfile: str) -> bool:
    """
    Function that checks if the shell is not ZSH and requests changes to it.
    """

    try:
        if shell() != "zsh":
            printer(
                "Changing the shell from Bash to ZSH (Root password required). [Press Ctrl+C to cancel]",
                foreground=FG().QUESTION,
            )
            if which("chsh"):
                call(f"chsh -s $(which zsh) {whoami()}", shell=True)
            elif which("usermod"):
                call(f'su -c "usermod -s $(which zsh) {whoami()}"', shell=True)
            else:
                return False
            return True
        return False
    except KeyboardInterrupt:
        Log(filename=logfile).record(
            f"Shell change canceled by user ({whoami()})",
            colorize=True,
            level="warning",
        )
        printer("Canceled by user.", foreground=FG().WARNING)
    return False


def open_file_with_editor(
    toml_file: str, file_common: str = "", superuser: bool = False
) -> None:
    """
    Opens file with a certain editor according to what is informed in the ZSHPower configuration file
    """

    def editor_run(editor: str, config: str, get_superuser: bool = superuser) -> bool:
        if which(editor):
            get_editor = os.environ.get("EDITOR", editor)
            with open(config) as f:
                # TODO: It is not accepting the "command_root" function. Solve.
                if get_superuser:
                    try:
                        cmd = f"""su -c '{get_editor} {f.name}';"""
                        printer(
                            "[ Enter the machine superuser password ]",
                            foreground=FG().WARNING,
                        )
                        call(cmd, shell=True, universal_newlines=True)
                        return True
                    except KeyboardInterrupt:
                        printer("Aborted by user.", foreground=FG().WARNING)
                        return False
                else:
                    call([get_editor, f.name])
                    return True
        return False

    def condition(editor: str):
        if file_common:
            return editor_run(editor, file_common, get_superuser=superuser)
        return editor_run(editor, toml_file, get_superuser=superuser)

    try:
        read_conf = read_file(toml_file)
        parsed = dict(toml_parse(read_conf))
        get_editor_name = get_key(parsed, "general", "config", "editor")
        if get_editor_name and get_editor_name != {}:
            condition(get_editor_name)
        else:
            editors: tuple = ("vim", "nano", "emacs", "micro", "vi")
            for edt in editors:
                if is_tool(edt):
                    condition(edt)
                    break
    except FileNotFoundError:
        printer(
            'ZSHPower task file does not exist in Cron. Use: "zshpower cron --create"',
            foreground=FG().WARNING,
        )



from os.path import exists, isdir
from pydoc import pager

from snakypy.helpers import FG, pick
from snakypy.helpers.catches import whoami
from snakypy.helpers.catches.finders import find_objects
from snakypy.helpers.console import printer
from snakypy.helpers.files import read_file
from snakypy.helpers.logging import Log

from snakypy.zshpower import __info__
from snakypy.zshpower.config.base import Base
from snakypy.zshpower.config.cron import cron_content
from snakypy.zshpower.utils.check import checking_init
from snakypy.zshpower.utils.modifiers import command_root
from snakypy.zshpower.utils.process import open_file_with_editor


class Cron(Base):
    def __init__(self, home: str):
        Base.__init__(self, home)

    def manager(self, action: str = "") -> bool:

        if not isdir(self.cron_d_path):
            printer(
                f'It appears that your machine does not have Cron installed. {__info__["name"]} '
                f"could not find the folder. Aborted!",
                foreground=FG().WARNING,
            )
            return False

        try:
            if action == "create":

                if exists(self.cron_path):
                    printer(
                        f"There is already a task in Cron for {__info__['name']}. Aborted!",
                        foreground=FG().WARNING,
                    )
                    return False

                printer(
                    f"Creating {__info__['name']} Task in Cron.",
                    foreground=FG().QUESTION,
                )
                cmd = f"""su -c 'echo "{cron_content}" > {self.cron_path}; chmod 644 {self.cron_path}'"""
                command_root(cmd, logfile=self.logfile)
                printer(
                    f"{__info__['name']} Cron task created!", foreground=FG().FINISH
                )

                printer(
                    f"""
                ************************************ WARNING *********************************************
                    The task was created, however you need to see if the Cron service is started and
                    edit the "{self.cron_path}" file, determining a time to be executed.
                    To edit, run the command: {FG().CYAN}${__info__["executable"]} cron --open{FG().YELLOW}
                ******************************************************************************************
                """,
                    foreground=FG().YELLOW,
                )

                return True

            elif action == "remove":
                cron_file = find_objects(
                    self.cron_d_path, files=(f"{__info__['pkg_name']}__{whoami()}",)
                )

                if not cron_file["files"]:
                    printer(
                        f'There is no {__info__["name"]} task file in Cron to remove. Aborted!',
                        foreground=FG().WARNING,
                    )
                    return False

                if cron_file["files"]:
                    title = f"Really want to remove {__info__['name']} task from Cron?"
                    options = ["Yes", "No"]
                    reply = pick(title, options, colorful=True, index=True)
                    cmd = f"""su -c 'rm -f {self.cron_path}';"""

                    if reply is None or reply[0] == 1:
                        printer("Canceled by user.", foreground=FG().WARNING)
                        return False

                    command_root(cmd, logfile=self.logfile)

                    printer(
                        f"{__info__['name']} task has been removed from Cron!",
                        foreground=FG().FINISH,
                    )
                    return True
            return False
        except PermissionError as err:
            Log(filename=self.logfile).record(
                "No permission to write to directory /etc/crond.d or /usr/local/bin.",
                colorize=True,
                level="error",
            )
            raise PermissionError(
                "No permission to write to directory /etc/crond.d or /usr/local/bin.",
                err,
            )

    def run(self, arguments: dict):
        checking_init(self.HOME, self.logfile)
        if arguments["--create"]:
            self.manager(action="create")
        elif arguments["--remove"]:
            self.manager(action="remove")
        elif arguments["--open"]:
            open_file_with_editor(
                self.config_file, file_common=self.cron_path, superuser=True
            )
        elif arguments["--view"]:
            try:
                read_config = read_file(self.cron_path)
                pager(read_config)
                return True
            except FileNotFoundError:
                printer(
                    f'ZSHPower task file does not exist in Cron. Use: "{__info__["executable"]} cron --create"',
                    foreground=FG().WARNING,
                )
            except PermissionError:
                printer(
                    "You do not have the necessary permission to view this file.",
                    foreground=FG().WARNING,
                )
            return False



import sys
from sqlite3 import OperationalError

from snakypy.helpers import FG
from snakypy.helpers.catches.generic import whoami
from snakypy.helpers.console import printer

from snakypy.zshpower.commands.utils.handle import records
from snakypy.zshpower.config.base import Base
from snakypy.zshpower.utils.check import checking_init


class Sync(Base):
    def __init__(self, home: str):
        Base.__init__(self, home)

    def run(self) -> None:
        try:
            checking_init(self.HOME, self.logfile)
            records("update", "Synchronizing versions with database ...", FG().QUESTION)
            self.log.record(
                f"User ({whoami()}) updated the database.",
                colorize=True,
                level="info",
            )
            printer("Done!", foreground=FG().FINISH)
        except KeyboardInterrupt:
            printer(
                "This operation cannot be canceled. Wait for the operation.",
                foreground=FG().WARNING,
            )
        except OperationalError:
            self.log.record(
                "The database does not exist or is corrupted.",
                colorize=True,
                level="error",
            )
            printer(
                "The database does not exist or is corrupted.",
                foreground=FG().ERROR,
            )
            sys.exit(1)



from os import environ, getcwd
from subprocess import CompletedProcess, check_output
from typing import List, Union

from snakypy.helpers.catches.finders import is_tool

from snakypy.zshpower.database.dao import DAO
from snakypy.zshpower.utils.catch import get_key, verify_objects


def symbol_ssh(symbol1: str, symbol2: str, spacing: str = " ") -> str:
    if symbol1 != {}:
        if symbol1 != "":
            symbol1 += spacing
        if "SSH_CONNECTION" in environ:
            symbol1 = symbol2
        return symbol1
    return ""


def git_status(
    *, porcelain: bool = False, branch: bool = False
) -> Union[str, List[str]]:
    porcelain_set = "--porcelain" if porcelain else ""
    branch_set = "--branch" if branch else ""

    if porcelain and not branch:
        status: Union[str, List[str]] = check_output(
            f"git status {porcelain_set}", shell=True, universal_newlines=True
        ).split()
    elif branch and not porcelain:
        data = check_output(
            f"git status {branch_set}", shell=True, universal_newlines=True
        ).split()
        if data[0] == "HEAD":
            status = f"{Color('red')}HEAD{Color().NONE}"
        else:
            status = data[2]
    else:
        status = check_output(
            f"git status {porcelain_set} {branch_set}",
            shell=True,
            universal_newlines=True,
        ).split()
    return status


def separator(config: dict, spacing: str = " ") -> str:

    sep = get_key(config, "general", "separator", "element")
    if sep is not dict:
        sep += spacing if sep != "" else sep
        sep_color = get_key(config, "general", "separator", "color")
        if sep_color is not dict:
            separator_style = f"{Color(sep_color)}{sep}{Color().NONE}"
            return separator_style
    return ""


def element_spacing(element: str, spacing: str = " "):
    if element != {}:
        if element != "":
            element += spacing
        return element
    return ""


class Color:
    """Color application compatible only ZSH."""

    NONE = "%f"

    def __init__(self, set_color: str = ""):
        self.color = f"%F{{{set_color}}}"
        if set_color == "negative":
            self.color = ""

    def __str__(self):
        return self.color


class Version(DAO):
    def __init__(self):
        DAO.__init__(self)
        self.finder_path = getcwd()
        self.finder = {"extensions": [], "folders": [], "files": []}

    def get(
        self,
        config: dict,
        database: dict,
        key: str = "",
        shorten: str = "",
        space_elem: str = "",
        not_split: bool = False,
    ) -> str:
        enable = get_key(config, key, "version", "enable")
        symbol = symbol_ssh(get_key(config, key, "symbol"), shorten)
        color = (
            get_key(config, key, "color")
            if get_key(config, "general", "color", "enable") is True
            else "negative"
        )
        prefix_color = get_key(config, key, "prefix", "color")
        prefix_text = element_spacing(get_key(config, key, "prefix", "text"))
        micro_version_enable = get_key(config, key, "version", "micro", "enable")

        if (
            enable is True
            and verify_objects(self.finder_path, data=self.finder) is True
        ):
            if get_key(database, key):
                prefix = f"{Color(prefix_color)}{prefix_text}{Color().NONE}"

                if not_split:
                    return str(
                        (
                            f"{separator(config)}{prefix}"
                            f"{Color(color)}{symbol}"
                            f"{get_key(database, key)}{space_elem}{Color().NONE}"
                        )
                    )

                if micro_version_enable is True:
                    version = f"{'{0[0]}.{0[1]}.{0[2]}'.format(get_key(database, key).split('.'))}{space_elem}"
                else:
                    version = f"{'{0[0]}.{0[1]}'.format(get_key(database, key).split('.'))}{space_elem}"

                return str(
                    (
                        f"{separator(config)}{prefix}"
                        f"{Color(color)}{symbol}"
                        f"{version}{Color().NONE}"
                    )
                )
        return ""

    def set(
        self,
        command: Union[CompletedProcess, CompletedProcess[str]],
        version: str,
        app_executable: str,
        key: str,
        action: str = "",
    ) -> bool:
        if is_tool(app_executable) and action:
            # Conditions to save in database
            if action == "insert":
                query = DAO().select_where(
                    self.tbl_main, key, "name", select=("version",)
                )
                if not query:
                    DAO().insert(
                        self.tbl_main,
                        columns=("name", "version"),
                        values=(key, version),
                    )

            elif action == "update":
                DAO().update(self.tbl_main, "version", version, "name", key)

            # Register logs
            if command.returncode != 0:
                self.log.record(
                    f"{key.title()} version not registered: {command.stderr}",
                    colorize=True,
                    level="error",
                )
            elif command.returncode == 0:
                self.log.record(
                    f"{key.title()} {version} registered in the database!",
                    colorize=True,
                    level="info",
                )

            return True
        return False



def sql() -> dict:
    data: dict = {
        "tbl_main": (
            "CREATE TABLE IF NOT EXISTS `tbl_main` ("
            "  id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,"
            "  name TEXT(20) NOT NULL,"
            "  version TEXT(20) NOT NULL,"
            "  date TIMESTAMP DEFAULT CURRENT_TIMESTAMP"
            ")"
        )
    }
    return data



from subprocess import run

from snakypy.zshpower.config.base import Base
from snakypy.zshpower.prompt.sections.utils import Version


class NodeJs(Version, Base):
    def __init__(self, *args):
        super(NodeJs, self).__init__()
        self.args: tuple = args
        self.key = "nodejs"
        self.app_executable = "node"
        self.shorten = "node-"
        self.finder = {
            "extensions": [".js"],
            "folders": ["node_modules"],
            "files": ["package.json"],
        }

    def get_version(self, space_elem: str = " ") -> str:
        # args[0]: dict = config file (toml)
        # args[1]: dict = database registers
        return super().get(
            self.args[0], self.args[1], self.key, self.shorten, space_elem=space_elem
        )

    def set_version(self, action: str = "") -> bool:
        command = run("node -v", capture_output=True, shell=True, text=True)
        version = command.stdout.replace("\n", "").split("v")[1]
        return super().set(
            command, version, self.app_executable, self.key, action=action
        )

    def __str__(self):
        return self.get_version()


# def _nodejs(config, key):
#     with concurrent.futures.ThreadPoolExecutor() as executor:
#         future = executor.submit(NodeJs().get_version, config, key)
#         return_value = future.result()
#         return return_value



from os import environ as os_environ
from socket import gethostname

from snakypy.zshpower.utils.catch import get_key

from .utils import Color, element_spacing, symbol_ssh


class Hostname:
    def __init__(self, config: dict):

        self.config: dict = config
        self.symbol = symbol_ssh(get_key(config, "hostname", "symbol"), "")
        self.hostname_enable = get_key(config, "hostname", "enable")
        self.hostname_color = get_key(config, "hostname", "color")
        self.hostname_prefix_color = get_key(config, "hostname", "prefix", "color")
        self.hostname_prefix_text = element_spacing(
            get_key(config, "hostname", "prefix", "text")
        )

    def __str__(self, prefix: str = "", space_elem: str = " ") -> str:
        if (
            get_key(self.config, "hostname", "enable") is True
            or "SSH_CONNECTION" in os_environ
        ):
            prefix = (
                f"{Color(self.hostname_prefix_color)}"
                f"{self.hostname_prefix_text}"
                f"{Color().NONE}"
            )

        if "SSH_CONNECTION" in os_environ or self.hostname_enable:
            return (
                f"{prefix}{Color(self.hostname_color)}{self.symbol}"
                f"{gethostname()}{space_elem}{Color().NONE}"
            )
        return ""



from subprocess import run

from snakypy.zshpower.config.base import Base
from snakypy.zshpower.prompt.sections.utils import Version


class Perl(Version, Base):
    def __init__(self, *args):
        super(Perl, self).__init__()
        self.args: tuple = args
        self.key = "perl"
        self.app_executable = "perl"
        self.shorten = "pl-"
        self.finder = {"extensions": [".pl"], "folders": [], "files": []}

    def get_version(self, space_elem: str = " ") -> str:
        return super().get(
            self.args[0], self.args[1], self.key, self.shorten, space_elem=space_elem
        )

    def set_version(self, action: str = "") -> bool:
        command = run(
            # Model
            # perl -Mstrict -wall -e "print join('.', map {ord} split('', \$^V));"
            # perl -v | awk '/This/ {print $4}' | sed -e 's/v//'
            f"{self.app_executable} -version | awk '/version/' 2>&1",
            capture_output=True,
            shell=True,
            text=True,
        )
        version = (
            command.stdout.split()[8].replace("v", "").replace("(", "").replace(")", "")
        )
        return super().set(
            command, version, self.app_executable, self.key, action=action
        )

    def __str__(self):
        return self.get_version()



from snakypy.zshpower import __info__

zshrc_content: str = f"""# Generate by: {__info__["name"]}
# If you come from bash you might have to change your $PATH.
# export PATH=$HOME/bin:/usr/local/bin:$PATH

# Path to your oh-my-zsh installation.
export ZSH="$HOME/.oh-my-zsh"

# Set name of the theme to load --- if set to "random", it will
# load a random theme each time oh-my-zsh is loaded, in which case,
# to know which specific one was loaded, run: echo $RANDOM_THEME
# See https://github.com/ohmyzsh/ohmyzsh/wiki/Themes
ZSH_THEME="{__info__['pkg_name']}"

# Set list of themes to pick from when loading at random
# Setting this variable when ZSH_THEME=random will cause zsh to load
# a theme from this variable instead of looking in ~/.oh-my-zsh/themes/
# If set to an empty array, this variable will have no effect.
# ZSH_THEME_RANDOM_CANDIDATES=( "robbyrussell" "agnoster" )

# Uncomment the following line to use case-sensitive completion.
# CASE_SENSITIVE="true"

# Uncomment the following line to use hyphen-insensitive completion.
# Case-sensitive completion must be off. _ and - will be interchangeable.
# HYPHEN_INSENSITIVE="true"

# Uncomment the following line to disable bi-weekly auto-update checks.
# DISABLE_AUTO_UPDATE="true"

# Uncomment the following line to automatically update without prompting.
# DISABLE_UPDATE_PROMPT="true"

# Uncomment the following line to change how often to auto-update (in days).
# export UPDATE_ZSH_DAYS=13

# Uncomment the following line if pasting URLs and other text is messed up.
# DISABLE_MAGIC_FUNCTIONS=true

# Uncomment the following line to disable colors in ls.
# DISABLE_LS_COLORS="true"

# Uncomment the following line to disable auto-setting terminal title.
# DISABLE_AUTO_TITLE="true"

# Uncomment the following line to enable command auto-correction.
# ENABLE_CORRECTION="true"

# Uncomment the following line to display red dots whilst waiting for
# completion.
# COMPLETION_WAITING_DOTS="true"

# Uncomment the following line if you want to disable marking untracked files
# under VCS as dirty. This makes repository status check for large
# repositories much, much faster.
# DISABLE_UNTRACKED_FILES_DIRTY="true"

# Uncomment the following line if you want to change the command execution
# time stamp shown in the history command output.
# You can set one of the optional three formats:
# "mm/dd/yyyy"|"dd.mm.yyyy"|"yyyy-mm-dd"
# or set a custom format using the strftime function format specifications,
# see 'man strftime' for details.
# HIST_STAMPS="mm/dd/yyyy"

# Would you like to use another custom folder than $ZSH/custom?
# ZSH_CUSTOM=/path/to/new-custom-folder

# Which plugins would you like to load?
# Standard plugins can be found in ~/.oh-my-zsh/plugins/*
# Custom plugins may be added to ~/.oh-my-zsh/custom/plugins/
# Example format: plugins=(rails git textmate ruby lighthouse)
# Add wisely, as too many plugins slow down shell startup.
plugins=(git python pip pep8 autopep8)

source $ZSH/oh-my-zsh.sh

# User configuration

# export MANPATH="/usr/local/man:$MANPATH"

# You may need to manually set your language environment
# export LANG=en_US.UTF-8

# Preferred editor for local and remote sessions
# if [[ -n $SSH_CONNECTION ]]; then
#   export EDITOR='vim'
# else
#   export EDITOR='mvim'
# fi

# Compilation flags
# export ARCHFLAGS="-arch x86_64"

# Set personal aliases, overriding those provided by oh-my-zsh libs,
# plugins, and themes. Aliases can be placed here, though oh-my-zsh
# users are encouraged to define aliases within the ZSH_CUSTOM folder.
# For a full list of active aliases, run `alias`.
#
# Example aliases
# alias zshconfig="mate ~/.zshrc"
# alias ohmyzsh="mate ~/.oh-my-zsh"
#
# PATH local user
PATH="$HOME/.local/bin:$PATH"
export PATH
    """

zshrc_sample: str = """
PATH="$PATH:$HOME/.local/bin:/usr/bin:/usr/local/bin"
export PATH

autoload -Uz compinit promptinit
compinit
promptinit
PROMPT='%F{green}%n%f@%F{magenta}%m%f %F{blue}%B%~%b%f %# '
"""



import re
from contextlib import suppress
from os import getcwd, listdir
from os.path import exists, isfile, join
from shutil import which
from subprocess import run

from snakypy.helpers.files import read_json
from snakypy.helpers.files.generic import read_file

from snakypy.zshpower.utils.catch import get_key, verify_objects

from .utils import Color, element_spacing, separator, symbol_ssh


class Base:
    def __init__(self, config: dict):
        self.config: dict = config
        self.finder: dict = {"extensions": [], "folders": [], "files": []}
        self.symbol = symbol_ssh(get_key(config, "package", "symbol"), "pkg-")
        self.enable = get_key(config, "package", "enable")
        self.color = (
            get_key(config, "package", "color")
            if get_key(config, "general", "color", "enable") is True
            else "negative"
        )
        self.prefix_color = get_key(config, "package", "prefix", "color")
        self.prefix_text = element_spacing(get_key(config, "package", "prefix", "text"))

    def finder_version(self, file, regex):
        try:
            if self.enable:
                file_content = read_file(file)
                re_search = re.search(regex, file_content)
                version = re_search.group(0)
                return version
            return ""
        except (IndexError, FileNotFoundError):
            return ""

    def get_version_yaml(self, space_elem=""):
        try:
            regex = r"version:.*"
            get_line = self.finder_version(self.finder["files"][0], regex)
            version = get_line.split(":")[1].strip()
            return f"{version}{space_elem}"
        except (IndexError, FileNotFoundError):
            return ""

    def get_version_toml(self, space_elem=""):
        try:
            regex = r"version =.*"
            get_line = self.finder_version(self.finder["files"][0], regex)
            version = get_line.split("=")[1].replace('"', "").replace("'", "").strip()
            return f"{version}{space_elem}"
        except (IndexError, FileNotFoundError):
            return ""

    def get_version_json(self, filename, space_elem=""):
        if self.enable and isfile(join(getcwd(), filename)):
            with suppress(Exception):
                parsed = read_json(join(getcwd(), filename))
                if "version" in parsed:
                    return f"{parsed['version']}{space_elem}"
        return ""

    def __str__(self, get_version=""):
        if self.enable:
            package_version = get_version
            if package_version and verify_objects(getcwd(), data=self.finder) is True:
                prefix = (
                    f"{Color(self.prefix_color)}" f"{self.prefix_text}{Color().NONE}"
                )
                return (
                    f"{separator(self.config)}{prefix}"
                    f"{Color(self.color)}"
                    f"{self.symbol}{package_version}{Color().NONE}"
                )
        return ""


class Python(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.finder = {
            "extensions": [],
            "folders": [],
            "files": [
                "pyproject.toml",
                "manage.py",
                "requirements.txt",
                "setup.py",
                "__init__.py",
            ],
        }

    def get_version(self, space_elem=" "):
        return super().get_version_toml(space_elem=space_elem)

    def __str__(self, get_version=""):
        return super().__str__(get_version=self.get_version())


class NodeJS(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.finder = {
            "extensions": [],
            "folders": ["node_modules"],
            "files": ["package.json"],
        }

    def get_version(self, space_elem=" "):
        return super().get_version_json(self.finder["files"][0], space_elem=space_elem)

    def __str__(self, get_version=""):
        return super().__str__(get_version=self.get_version())


class Rust(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.finder = {"extensions": [], "folders": [], "files": ["Cargo.toml"]}

    def get_version(self, space_elem=" "):
        return super().get_version_toml(space_elem=space_elem)

    def __str__(self, get_version=""):
        return super().__str__(get_version=self.get_version())


class Scala(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.finder = {"extensions": [], "folders": [], "files": ["build.sbt"]}

    def get_version(self, space_elem=" "):
        try:
            regex = r"version := .*"
            get_line = super().finder_version(self.finder["files"][0], regex)
            version = get_line.split('"')[1]
            return f"{version}{space_elem}"
        except (IndexError, FileNotFoundError):
            return ""

    def __str__(self, get_version=""):
        return super().__str__(get_version=self.get_version())


class Crystal(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.finder = {"extensions": [], "folders": [], "files": ("shard.yml",)}

    def get_version(self, space_elem=" "):
        return super().get_version_yaml(space_elem=space_elem)

    def __str__(self, get_version=""):
        return super().__str__(get_version=self.get_version())


class Helm(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.finder = {"extensions": [], "folders": [], "files": ["Chart.yaml"]}

    def get_version(self, space_elem=" "):
        return super().get_version_yaml(space_elem=space_elem)

    def __str__(self, get_version=""):
        return super().__str__(get_version=self.get_version())


class Ruby(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.finder = {"extensions": [".gemspec"], "folders": [], "files": []}

    def search_gemspec(self, directory):
        for file in listdir(directory):
            if file.endswith(self.finder["extensions"][0]):
                return file
        return False

    def get_version(self, space_elem=" "):
        file = self.search_gemspec(getcwd())
        if file is not False and which("ruby"):
            command = run(
                f"""ruby -e 'puts Gem::Specification::load("{file}").version'""",
                shell=True,
                text=True,
                capture_output=True,
            )
            if command.returncode == 0:
                version = command.stdout.strip().replace("\n", "")
                return f"{version}{space_elem}"
        return ""

    def __str__(self, get_version=""):
        return super().__str__(get_version=self.get_version())


class V(Base):
    def __init__(self, config):
        Base.__init__(self, config)
        self.finder = {"extensions": [], "folders": [], "files": ["vpkg.json"]}

    def get_version(self, space_elem=" "):
        return super().get_version_yaml(space_elem=space_elem)

    def __str__(self, get_version=""):
        return super().__str__(get_version=self.get_version())


class Package:
    def __init__(self, config: dict, *args):
        self.config: dict = config
        self.args = args

    def __str__(self):
        listing = get_key(self.config, "package", "display")

        if listing:

            pyproject_toml = join(getcwd(), Python(self.config).finder["files"][0])
            gemspec = Ruby(self.config).search_gemspec(getcwd())
            package_json = join(getcwd(), NodeJS(self.config).finder["files"][0])
            cargo_toml = join(getcwd(), Rust(self.config).finder["files"][0])
            build_sbt = join(getcwd(), Scala(self.config).finder["files"][0])
            shard_yaml = join(getcwd(), Crystal(self.config).finder["files"][0])
            chart_yaml = join(getcwd(), Helm(self.config).finder["files"][0])
            vpkg_json = join(getcwd(), V(self.config).finder["files"][0])

            if exists(pyproject_toml) and "python" in listing:
                return str(Python(self.config))
            elif exists(package_json) and ("node" in listing or "nodejs" in listing):
                return str(NodeJS(self.config))
            elif gemspec is not False and "ruby" in listing:
                return str(Ruby(self.config))
            elif exists(cargo_toml) and "rust" in listing:
                return str(Rust(self.config))
            elif exists(build_sbt) and "scala" in listing:
                return str(Scala(self.config))
            elif exists(chart_yaml) and "helm" in listing:
                return str(Helm(self.config))
            elif exists(vpkg_json) and "v" in listing:
                return str(V(self.config))
            elif exists(shard_yaml) and "crystal" in listing:
                return str(Crystal(self.config))
            elif exists(chart_yaml) and "helm" in listing:
                return str(Helm(self.config))
        return ""



from getpass import getpass
from os import remove
from os.path import exists, isdir, isfile, join
from re import M, sub
from shutil import which
from subprocess import PIPE, Popen, check_output
from sys import exit, platform
from typing import Iterable
from zipfile import ZipFile

from snakypy.helpers import FG, printer
from snakypy.helpers.files import backup_file, create_file
from snakypy.helpers.logging import Log
from snakypy.helpers.path import create as create_path
from snakypy.helpers.subprocess import command
from tomlkit import dumps as toml_dumps
from tomlkit import parse as toml_parse

from snakypy.zshpower.utils.catch import current_plugins, get_zsh_theme, read_file_log


def create_toml(content: str, file: str, *, force=False) -> bool:
    """
    Create the ZSHPower configuration file. A TOML file.
    """
    if not exists(file) or force:
        parsed_toml = toml_parse(content)
        write_toml = toml_dumps(parsed_toml)
        create_file(write_toml, file, force=force)
        return True
    return False


def create_zshrc(content: str, zshrc_path: str, logfile: str) -> None:
    """
    Create a .zshrc file if there is no one compatible with Oh MyZSH.
    """
    try:
        if not get_zsh_theme(zshrc_path, logfile):
            backup_file(zshrc_path, zshrc_path, date=True, extension=False)
            create_file(content, zshrc_path, force=True)
    except FileNotFoundError:
        create_file(content, zshrc_path)


def command_root(
    cmd: str, logfile: str = "", msg_header: str = "Enter the machine root password"
) -> bool:
    check = False
    printer(f"[ {msg_header} ]", foreground=FG().WARNING)
    try:
        while not check:
            sudo_password = getpass()
            popen = Popen(
                cmd,
                stdin=PIPE,
                stderr=PIPE,
                stdout=PIPE,
                universal_newlines=True,
                shell=True,
            )
            communicate = popen.communicate(sudo_password)
            if "su:" in communicate[1].split():
                printer("Password incorrect.", foreground=FG().ERROR)
            else:
                check = True
        return check
    except KeyboardInterrupt:
        printer("Aborted by user.", foreground=FG().WARNING)
        exit()
    except PermissionError:
        if logfile:
            Log(filename=logfile).record(
                "No permission to write to directory.", colorize=True, level="error"
            )
        raise PermissionError("No permission to write to directory")


def change_theme(file: str, theme_name: str, logfile: str) -> bool:
    """
    Change Oh My ZSH Theme
    """
    if get_zsh_theme(file, logfile):
        current_file = read_file_log(file, logfile)
        current_theme = get_zsh_theme(file, logfile)[1]
        new_theme = f'ZSH_THEME="{theme_name}"'
        new_file = sub(rf"{current_theme}", new_theme, current_file, flags=M)
        create_file(new_file, file, force=True)
        return True
    return False


def omz_install(omz_root: str, logfile: str) -> bool:
    """
    Install Oh My ZSH
    """
    omz_github = "https://github.com/ohmyzsh/ohmyzsh.git"
    cmd_line = f"git clone {omz_github} {omz_root}"
    try:
        if not exists(omz_root):
            printer("Install Oh My ZSH...", foreground=FG().QUESTION)
            command(cmd_line, verbose=True)
            printer(
                "Oh My ZSH installation process finished.",
                foreground=FG().FINISH,
            )
            return True
        return False
    except Exception:
        Log(filename=logfile).record(
            "Error downloading Oh My ZSH. Aborted!", colorize=True, level="error"
        )
        raise Exception("Error downloading Oh My ZSH. Aborted!")


def install_plugins(omz_root: str, plugins: Iterable[tuple], logfile: str):
    """
    Install plugins on Oh My ZSH
    """
    try:
        url_master = "https://github.com/zsh-users"
        for plugin in plugins:
            path = join(omz_root, f"custom/plugins/{plugin}")
            clone = f"git clone {url_master}/{plugin}.git {path}"
            if not isdir(path):
                printer(f"Install plugins {plugin}...", foreground=FG().QUESTION)
                command(clone, verbose=True)
                printer(f"Plugin {plugin} task finished!", foreground=FG().FINISH)
    except Exception:
        Log(filename=logfile).record(
            "There was an error installing the plugin", colorize=True, level="error"
        )
        raise Exception("There was an error installing the plugin")


def install_fonts(home: str, logfile: str, *, force: bool = False) -> bool:
    """
    Install the Nerd Fonts font in the $HOME/.fonts folder
    """
    url = "https://github.com/snakypy/assets"
    base_url = "raw/main/zshpower/fonts/terminal/fonts.zip"
    font_name = "DejaVu Sans Mono Nerd Font"
    fonts_dir = join(home, ".fonts")
    create_path(fonts_dir)
    curl_output = join(home, "zshpower__font.zip")

    if platform.startswith("linux"):
        try:
            if (
                not isfile(join(fonts_dir, "DejaVu Sans Mono Nerd Font Complete.ttf"))
                or force
            ):
                printer(
                    f'Please wait, downloading the "{font_name}" font and'
                    "installing...",
                    foreground=FG().QUESTION,
                )
                cmd_line = f"curl -L {join(url, base_url)} -o {curl_output}"
                command(cmd_line, verbose=True)

                with ZipFile(curl_output, "r") as zip_ref:
                    zip_ref.extractall(fonts_dir)
                    remove(curl_output)
                    printer("Done!", foreground=FG().FINISH)
                return True
            return False
        except Exception as err:
            Log(filename=logfile).record(
                f'Error downloading font "{font_name}"', colorize=True, level="error"
            )
            raise Exception(f'Error downloading font "{font_name}"', err)
    return False


def add_plugins(zshrc, logfile):
    """
    Add plugins to the .zshrc file if it is compatible with Oh My ZSH
    """
    plugins = (
        "python",
        "pip",
        "pep8",
        "autopep8",
        "virtualenv",
        "zsh-syntax-highlighting",
        "zsh-autosuggestions",
    )
    current = current_plugins(zshrc, logfile)

    new_plugins = [plugin for plugin in plugins if plugin not in current]

    if len(new_plugins) > 0:
        current_zshrc = read_file_log(zshrc, logfile)
        plugins = f'plugins=({" ".join(current)} {" ".join(new_plugins)})'
        new_zsh_rc = sub(r"^plugins=\(.*", plugins, current_zshrc, flags=M)
        create_file(new_zsh_rc, zshrc, force=True)
        return new_zsh_rc
    return ""


def remove_lines(file: str, logfile: str, lines: tuple = ()) -> None:
    """
    Remove certain lines from a file
    """
    content = read_file_log(file, logfile)
    for num, _ in enumerate(lines):
        content = sub(rf"{lines[num]}", "", content, flags=M)

    create_file(content, file, force=True)


def pip_uninstall(*, packages: tuple = ()) -> tuple:
    """
    Install Python packages for active user using Pip
    """
    if which("pip") is not None:
        for pkg in packages:
            check_output(
                f"pip uninstall {pkg} -y",
                shell=True,
                universal_newlines=True,
            )
    return packages



from snakypy.helpers import printer
from snakypy.helpers.ansi import FG
from snakypy.helpers.catches.generic import whoami

from snakypy.zshpower.commands.utils.handle import records
from snakypy.zshpower.config.base import Base
from snakypy.zshpower.config.config import config_content
from snakypy.zshpower.database.dao import DAO
from snakypy.zshpower.utils.check import checking_init
from snakypy.zshpower.utils.modifiers import create_toml
from snakypy.zshpower.utils.process import reload_zsh


class ResetCommand(Base):
    def __init__(self, home: str):
        Base.__init__(self, home)

    def run(self, arguments: dict) -> None:
        checking_init(self.HOME, self.logfile)
        if arguments["--config"]:
            create_toml(config_content, self.config_file, force=True)
            printer("Reset process finished.", foreground=FG().FINISH)
            self.log.record(
                f"User ({whoami()}) reset settings.", colorize=True, level="info"
            )
            reload_zsh()
        elif arguments["--db"]:
            DAO().create_table(self.tbl_main)
            records("insert", "ZSHPower Restoring the database ...", FG().QUESTION)
            printer("Done!", foreground=FG().FINISH)
            self.log.record(
                f"User ({whoami()}) reset database.", colorize=True, level="info"
            )



from snakypy.zshpower import __info__

bootstrap = f"""#!/usr/bin/env zsh

## Information:
## ******************************************************************************
## Theme: {__info__["name"]}
## Description: Call script to start {__info__["name"]}.
## ******************************************************************************

## Export PATH local user
export PATH="$PATH:$HOME/.local/bin"

function preexec() {{
  timer=$(date +%s)
}}

function precmd() {{
    if [ $timer ]; then
      now=$(date +%s)
      elapsed=$(( $now - $timer ))
      if [[ $elapsed -ge 60 ]]; then
        took=$(date -d@$elapsed -u "+%Mm %Ss")
      else
        took=$(date -d@$elapsed -u "+%Ss")
      fi
      unset timer
    fi
}}

## Option using "add-zsh-hook"
function zshpower_precmd() {{
  state=$(which zshpower-draw > /dev/null 2>&1)
  if [ ! $? -ne 0 ]; then
    PROMPT="$(zshpower-draw prompt $took)"
    RPROMPT="$(zshpower-draw rprompt)"
  else
    PROMPT='%F{{green}}%n%f@%F{{magenta}}%m%f %F{{blue}}%B%~%b%f %# '
    RPROMPT='[%F{{yellow}}%?%f]'
  fi
}}

function install_zshpower_precmd() {{
  for s in "${{precmd_functions[@]}}"; do
    if [ "$s" = "zshpower_draw" ]; then
      return
    fi
  done
  precmd_functions+=(zshpower_precmd)
}}

## Option using "add-zsh-hook"
autoload -Uz add-zsh-hook
add-zsh-hook precmd zshpower_precmd

## Option not using "add-zsh-hook"
# if [ "$TERM" != "linux" ]; then
#    install_zshpower_precmd
# fi
"""



from subprocess import run

from snakypy.zshpower.config.base import Base
from snakypy.zshpower.prompt.sections.utils import Version


class R(Version, Base):
    def __init__(self, *args):
        super(R, self).__init__()
        self.args: tuple = args
        self.key = "r"
        self.app_executable = "Rscript"
        self.shorten = "r-"
        self.finder = {"extensions": [".r", ".R"], "folders": [], "files": []}

    def get_version(self, space_elem: str = " ") -> str:
        # args[0]: dict = config file (toml)
        # args[1]: dict = database registers
        return super().get(
            self.args[0],
            self.args[1],
            self.key,
            self.shorten,
            space_elem=space_elem,
            not_split=True,
        )

    def set_version(self, action: str = "") -> bool:
        command = run(
            f"{self.app_executable} --version 2>&1",
            capture_output=True,
            shell=True,
            text=True,
        )
        version = command.stdout.strip().replace("\n", "").split(" ")[-2]
        return super().set(
            command, version, self.app_executable, self.key, action=action
        )

    def __str__(self):
        return self.get_version()



from snakypy.helpers.ansi import FG, NONE

from snakypy.zshpower import __info__

dt_omz: str = f"{FG().MAGENTA}robbyrussell{NONE}"
omz: str = f"{FG().MAGENTA}Oh My ZSH{NONE}"

options: str = f"""
{FG().MAGENTA}Welcome to the {__info__["pkg_name"]} options menu.{NONE}

Usage:
    {__info__['executable']} init [--omz | --path]
    {__info__['executable']} sync
    {__info__['executable']} cron (--create | --remove | --open | --view)
    {__info__['executable']} config (--open | --view)
    {__info__['executable']} logs (--view | --clean)
    {__info__['executable']} activate
    {__info__['executable']} deactivate [--theme=<name>]
    {__info__['executable']} reset (--config | --db)
    {__info__['executable']} uninstall
    {__info__['executable']} --help
    {__info__['executable']} --version
    {__info__['executable']} --credits

Positional arguments:
    {FG().CYAN}init{NONE} ---------- Installs the {__info__["name"]} settings.
    {FG().CYAN}sync{NONE} ---------- Synchronizes the machine's data with the {__info__["name"]} database.
    {FG().CYAN}cron{NONE} ---------- Manage {__info__["name"]} Tasks in Cron.
    {FG().CYAN}activate{NONE} ------ Activate the {__info__["name"]} theme. ({omz}).
    {FG().CYAN}deactivate{NONE} ---- Deactivate the {__info__["name"]} theme and go back to the default. ({omz}).
    {FG().CYAN}reset{NONE} --------- Reset to default settings.
    {FG().CYAN}uninstall{NONE} ----- Uninstall the package {__info__["name"]}.
    {FG().CYAN}config{NONE} -------- The easiest way to edit and view the settings is through this option.
    {FG().CYAN}logs{NONE} ---------- Shows {__info__["name"]} activity logs.
    {FG().CYAN}--version{NONE} ------ Show version.
    {FG().CYAN}--credits{NONE} ------ Show credits.
    {FG().CYAN}--help{NONE} --------- Show this screen.

Options:
    {FG().CYAN}--omz{NONE} ---------- Use this option if you want to use {__info__["name"]} with Oh My ZSH.
    {FG().CYAN}--open{NONE} --------- Open the configuration file in edit mode and perform the automatic update
                     when you exit.
    {FG().CYAN}--path{NONE} --------- View the path of the ZSHPower main lib.
    {FG().CYAN}--view{NONE} --------- View the configuration file on the terminal.
    {FG().CYAN}--config{NONE} ------- Restores the configuration file.
    {FG().CYAN}--create{NONE} ------- Create an object on the machine or some configuration.
    {FG().CYAN}--remove{NONE} ------- Remove an object on the machine or some configuration.
    {FG().CYAN}--db{NONE} ----------- Apply actions to the database.
    {FG().CYAN}--theme=<name>{NONE} - Get the name of a theme available on Oh My ZSH [Default: {dt_omz}].
"""



from os import environ as os_environ
from os import geteuid

from snakypy.helpers.catches import whoami

from snakypy.zshpower.utils.catch import get_key

from .utils import Color, symbol_ssh


class Username:
    def __init__(self, config: dict):
        self.symbol = symbol_ssh(get_key(config, "username", "symbol"), "")
        self.enable = get_key(config, "username", "enable")
        self.color = get_key(config, "username", "color")
        if geteuid() == 0:
            self.color = "red"

    def __str__(self, space_elem: str = " "):
        if self.enable is True or "SSH_CONNECTION" in os_environ or geteuid() == 0:
            return (
                f"{Color(self.color)}{self.symbol}{whoami()}{space_elem}{Color().NONE}"
            )
        return ""



from pydoc import pager

from snakypy.helpers.files import read_file

from snakypy.zshpower.config.base import Base
from snakypy.zshpower.utils.check import checking_init
from snakypy.zshpower.utils.process import open_file_with_editor


class ConfigCommand(Base):
    def __init__(self, home: str):
        Base.__init__(self, home)

    def run(self, arguments: dict) -> bool:
        checking_init(self.HOME, self.logfile)

        if arguments["--open"]:
            open_file_with_editor(self.config_file)
        elif arguments["--view"]:
            read_config = read_file(self.config_file)
            pager(read_config)
            return True
        return False



from snakypy.zshpower.utils.catch import get_key

from .utils import Color, symbol_ssh


class Command:
    def __init__(self, config: dict, *args):
        self.config: dict = config
        self.args = args
        self.symbol = symbol_ssh(get_key(config, "command", "symbol"), "> ")
        self.error_symbol = symbol_ssh(
            get_key(config, "command", "error", "symbol"), "x "
        )
        self.error_color = get_key(config, "command", "error", "color")
        self.color = (
            get_key(config, "command", "color")
            if get_key(config, "general", "color", "enable") is True
            else "negative"
        )
        self.new_line = get_key(config, "command", "new_line", "enable")

    def __str__(self, jump_line: str = "\n"):
        if not self.new_line:
            jump_line = ""
        return (
            f"{jump_line}{Color(self.color)}"
            f"%(?.{self.symbol}."
            f"{Color(self.error_color)}"
            f"{self.error_symbol})"
            f"{Color().NONE}"
        )



from contextlib import suppress
from os import getcwd
from os.path import join

from snakypy.helpers.files import read_json

from snakypy.zshpower.config.base import Base
from snakypy.zshpower.prompt.sections.utils import (
    Color,
    Version,
    element_spacing,
    separator,
    symbol_ssh,
)
from snakypy.zshpower.utils.catch import get_key, verify_objects


class Ember(Version, Base):
    def __init__(self, *args):
        super(Ember, self).__init__()
        self.args: tuple = args
        self.key = "ember"
        self.shorten = "ember-"
        self.finder = {
            "extensions": [],
            "folders": ("node_modules",),
            "files": [
                join("node_modules", "ember-cli", "package.json"),
                "ember-cli-build.js",
            ],
        }

    def get_version(self, space_elem: str = " "):
        enable = get_key(self.args[0], self.key, "version", "enable")
        symbol = symbol_ssh(get_key(self.args[0], self.key, "symbol"), self.shorten)
        color = (
            get_key(self.args[0], self.key, "color")
            if get_key(self.args[0], "general", "color", "enable") is True
            else "negative"
        )
        prefix_color = get_key(self.args[0], self.key, "prefix", "color")
        prefix_text = element_spacing(get_key(self.args[0], self.key, "prefix", "text"))
        micro_version_enable = get_key(
            self.args[0], self.key, "version", "micro", "enable"
        )

        if enable is True and verify_objects(getcwd(), data=self.finder) is True:
            with suppress(FileNotFoundError):
                parsed = read_json(self.finder["files"][0])
                if "version" not in parsed:
                    return ""
                version = f"{parsed['version']}{space_elem}"

                # # Using subprocess
                # command = run(
                #     f"""grep '"version":' {self.files[1]} | cut -d\\" -f4""",
                #     capture_output=True,
                #     shell=True,
                #     text=True,
                # )
                # version = command.stdout.replace("\n", "").strip()

                version = version.replace("\n", "").strip()

                prefix = f"{Color(prefix_color)}{prefix_text}{Color().NONE}"

                if micro_version_enable is True:
                    version = f"{'{0[0]}.{0[1]}.{0[2]}'.format(version.split('.'))}"
                else:
                    version = f"{'{0[0]}.{0[1]}'.format(version.split('.'))}"

                return str(
                    (
                        f"{separator(self.args[0])}{prefix}"
                        f"{Color(color)}{symbol}"
                        f"{version}{space_elem}{Color().NONE}"
                    )
                )
        return ""

    def __str__(self):
        return self.get_version()



# from subprocess import call
#
# from snakypy.helpers.ansi import FG
# from snakypy.helpers.console.display import printer
#
# from snakypy.zshpower.config.base import Base
# from snakypy.zshpower.utils.catch import get_zsh_theme
# from snakypy.zshpower.utils.check import checking_init
#
# # TODO: It will still be decided whether to implement
# class Theme(Base):
#     def __init__(self, home):
#         Base.__init__(self, home)
#
#     def run(self, arguments):
#         checking_init(self.HOME, self.logfile)
#
#         zshrc_omz = get_zsh_theme(self.zsh_rc, self.logfile)
#
#         if not zshrc_omz:
#             return printer(
#                 "Command not available. Only with Oh My ZSH installed. Aborted.",
#                 foreground=FG().WARNING,
#             )
#
#         if zshrc_omz and arguments["--list"]:
#             call(["zsh", "-i", "-c", "omz theme list"])



from pydoc import pager

from snakypy.helpers import printer
from snakypy.helpers.ansi import FG
from snakypy.helpers.files import create_file, read_file
from snakypy.helpers.os.removals import remove_objects

from snakypy.zshpower.config.base import Base
from snakypy.zshpower.utils.check import checking_init, is_blank_file


class LogsCommand(Base):
    def __init__(self, home: str):
        Base.__init__(self, home)

    def run(self, arguments: dict) -> None:
        checking_init(self.HOME, self.logfile)
        if arguments["--view"]:
            try:
                read_logs = read_file(self.logfile)
                pager(read_logs)
            except FileNotFoundError:
                printer("Log files not found:", foreground=FG().WARNING)
        elif arguments["--clean"]:
            if is_blank_file(self.logfile):
                printer(
                    "The logs have already been cleared. Nothing to do.",
                    foreground=FG().WARNING,
                )
            else:
                remove_objects(objects=(self.logfile,))
                create_file("", self.logfile, force=True)
                printer("Logs have been cleaned up.", foreground=FG().FINISH)



from contextlib import suppress
from os import remove, symlink
from os.path import islink, join
from sys import stdout

from snakypy.helpers import printer
from snakypy.helpers.ansi import FG, NONE
from snakypy.helpers.files import backup_file, create_file
from snakypy.helpers.os import remove_objects
from snakypy.helpers.path import create as create_path

from snakypy.zshpower import __info__
from snakypy.zshpower.commands.utils.handle import records
from snakypy.zshpower.config.base import Base
from snakypy.zshpower.config.bootstrap import bootstrap
from snakypy.zshpower.config.config import config_content
from snakypy.zshpower.config.zshrc import zshrc_content, zshrc_sample
from snakypy.zshpower.database.dao import DAO
from snakypy.zshpower.utils.catch import get_line, get_zsh_theme
from snakypy.zshpower.utils.check import tools_requirements
from snakypy.zshpower.utils.modifiers import (
    add_plugins,
    change_theme,
    create_toml,
    create_zshrc,
    install_fonts,
    install_plugins,
    omz_install,
    remove_lines,
)
from snakypy.zshpower.utils.process import change_shell, reload_zsh


class InitCommand(Base):
    def __init__(self, home: str):
        Base.__init__(self, home)
        self.instruction_not_omz = f"""{FG().YELLOW}
            **************************** WARNING *******************************
            1- Add the following line of code to the {FG().MAGENTA}{home}/.zshrc{NONE}{FG().YELLOW} file:

            {FG().CYAN}eval "$(zshpower init --path)"{NONE}

            {FG().YELLOW}2 - Then run the following command: {FG().CYAN}exec zsh{NONE}{FG().YELLOW}
            ********************************************************************{NONE}
        """

    def run(self, arguments: dict, *, reload: bool = False) -> None:
        tools_requirements("bash", "zsh", "vim", "git", "cut", "grep", "whoami", "pwd")

        # If there is, an exception will not pop.
        with suppress(FileExistsError):
            create_file(zshrc_sample, self.zsh_rc)

        # Returns only the path
        if arguments["--path"]:
            stdout.write(
                join(f'[[ -d "{self.lib_root}" ]] && source $HOME', self.source_code)
            )
        else:
            printer(
                "Wait a moment, creating initial settings...", foreground=FG().QUESTION
            )
            create_path(
                self.config_root, self.database_root, self.cache_root, self.lib_root
            )
            create_toml(config_content, self.config_file)
            create_file(bootstrap, self.lib_main, force=True)
            # Install with OMZ
            if arguments["--omz"]:
                remove_lines(
                    self.zsh_rc,
                    self.logfile,
                    lines=('eval "\\$\\(zshpower init --path\\)"',),
                )
                omz_install(self.omz_root, self.logfile)
                install_plugins(self.omz_root, self.plugins, self.logfile)
                create_zshrc(zshrc_content, self.zsh_rc, self.logfile)
                change_theme(self.zsh_rc, f"{__info__['pkg_name']}", self.logfile)
                add_plugins(self.zsh_rc, self.logfile)
                if islink(self.theme_symlink):
                    remove(self.theme_symlink)
                symlink(self.lib_main, self.theme_symlink)
            # Install fonts
            install_fonts(self.HOME, self.logfile)
            # Changing shell to ZSH
            change_shell(self.logfile)
            printer("Settings finished!", foreground=FG().FINISH)

            # printer("Generating database, wait...", foreground=FG().QUESTION)
            # Create table in database if not exists
            DAO().create_table(self.tbl_main)
            # Insert registers in database
            records("insert", "Generating database, wait ...", FG().QUESTION)
            printer("Database generated!", foreground=FG().FINISH)

            # Register logs
            self.log.record("Initial settings applied", colorize=True, level="info")

            # Create new .zshrc and Instruction ZSHPower without OMZ
            if not arguments["--omz"]:
                if get_zsh_theme(self.zsh_rc, self.logfile):
                    backup_file(self.zsh_rc, self.zsh_rc, date=True, extension=False)
                    remove_objects(objects=(self.zsh_rc,))
                    create_file(zshrc_sample, self.zsh_rc, force=True)
                line = 'eval "\\$\\(zshpower init --path\\)"'
                if not get_line(self.zsh_rc, line, self.logfile):
                    printer(self.instruction_not_omz, foreground=FG().YELLOW)

            # Reload terminal
            if arguments["--omz"] and reload:
                reload_zsh(sleep_timer=2, message=True)



from os.path import exists, getsize, isfile, join
from shutil import which
from sys import exit

from snakypy.helpers import FG, printer
from snakypy.helpers.decorators import only_linux
from snakypy.helpers.logging import Log

from snakypy.zshpower import __info__


def is_blank_file(filepath: str) -> bool:
    """
    Checks if a file is blank, that is, it does not contain lines.
    """
    return isfile(filepath) and getsize(filepath) == 0


@only_linux
def tools_requirements(*args) -> bool:
    """
    Function that looks for the necessary tools, if it doesn't find them,
    it triggers a message and closes the program.
    """
    for tool in args:
        if which(tool) is None:
            printer(
                f'The tool "{tool}" is not installed on the operating system.',
                foreground=FG().ERROR,
            )
            exit()
    return True


def checking_init(home: str, logfile: str) -> bool:
    """
    Function that ends commands that depend on the created repository, but
    the repository was not created.
    """

    if not exists(join(home, f".{__info__['pkg_name']}")):
        Log(filename=logfile).record(
            f'Command "{__info__["pkg_name"]} init" has not been started.' "Aborted",
            colorize=True,
            level="error",
        )
        raise FileNotFoundError(
            f'Command "{__info__["pkg_name"]} init" has not been started.' "Aborted",
        )
    return True


def str_empty_in(*args) -> bool:
    """
    Function that checks whether an object is an empty string.

    Returns:
        [bool]: Return bool
    """
    for _ in args:
        if "" in args:
            return True
    return False



from functools import reduce
from os import listdir
from os.path import join, splitext
from re import M
from re import search as re_search
from typing import Any, Union

from docopt import docopt
from snakypy.helpers.ansi import FG, NONE
from snakypy.helpers.logging import Log

from snakypy.zshpower import __info__
from snakypy.zshpower.config import menu


def get_key(dictionary: dict, *args) -> Union[str, bool, dict]:
    """
    Function to get keys from a dictionary recursively without errors.
    If the key does not exist it returns an empty dictionary.

    Args:
        dictionary ([dict]): Receive a dictionary

    Returns:
        [str, bool]: Returning a str or a boolean if the object is True or False.
    """
    data = reduce(lambda c, k: c.get(k, {}), args, dictionary)
    if data == {}:
        return ""
    return data


def read_file_log(file: str, logfile: str) -> str:
    """
    Read a file with log creation option if it does not exist
    """
    try:
        with open(file) as f:
            return f.read()
    except FileNotFoundError as err:
        Log(filename=logfile).record(
            f"The {file} file was not found.", colorize=True, level="error"
        )
        raise FileNotFoundError(f"The {file} file was not found.", err)


def arguments(argv: Any = None) -> dict:
    formatted_version = (
        f"{__info__['name']} version: {FG().CYAN}{__info__['version']}{NONE}"
    )
    data = docopt(menu.options, argv=argv, version=formatted_version)
    return data


def get_zsh_theme(file: str, logfile: str) -> Union[tuple, bool]:
    """
    Get the current theme contained in the .zshrc file if using Oh My ZSH
    """
    current_file = read_file_log(file, logfile)
    m = re_search(r"ZSH_THEME=\".*", current_file)
    if m is not None:
        theme_var = m.group(0)
        lst = theme_var.split("=")
        theme_name = [s.strip('"') for s in lst][1]
        return theme_name, theme_var
    return False


def current_plugins(file: str, logfile: str) -> list:
    """
    Get current plugins from .zshrc file if using Oh My ZSH
    """
    current_file = read_file_log(file, logfile)
    m = re_search(r"^plugins=\(.*", current_file, flags=M)
    if m is not None:
        get = m.group(0)
        lst = get.split("=")
        current = [i.strip('"').replace("(", "").replace(")", "") for i in lst][1]
        return current.split()
    return []


def get_line(file: str, line: str, logfile: str) -> Union[str, bool]:
    """
    Get a given line from a given file
    """
    current_file = read_file_log(file, logfile)
    m = re_search(rf"{line}", current_file)
    if m is not None:
        return m.group(0)
    return False


# TODO: DEPRECATED
# def verify_objects(directory: str, /, data: dict, strictly: bool = False) -> bool:
#     """
#     Checks whether there are objects from a particular directory.
#     These can be folders, files and file extensions.
#     """
#     if type(data) is not dict:
#         raise TypeError("Data parameter must be a dictionary.")
#
#     if "files" not in data or "folders" not in data or "extensions" not in data:
#         raise TypeError(
#             "The dictionary does not contain one of the following keys: 'files', 'folders', 'extensions'."
#         )
#
#     def finder_files_folders():
#         if data["files"] or data["folders"]:
#             files_folders = data["files"] + data["folders"]
#             for obj in listdir(directory):
#                 if strictly is False:
#                     if obj in files_folders:
#                         return True
#                 else:
#                     if obj not in files_folders:
#                         return False
#
#         return False
#
#     def finder_extensions():
#         if data["extensions"]:
#             lst = []
#             for ext in data["extensions"]:
#                 for file in listdir(directory):
#                     if file.endswith(ext):
#                         lst.append(join(directory, file))
#
#             e = [splitext(f)[-1] for f in lst]
#             for ext in data["extensions"]:
#                 if ext in e:
#                     return True
#         return False
#
#     if finder_files_folders() is False and finder_extensions() is False:
#         return False
#
#     return True
#


def verify_objects(directory: str, /, data: dict) -> bool:
    """
    Checks whether there are objects from a particular directory.
    These can be folders, files and file extensions.
    """
    if type(data) is not dict:
        raise TypeError("Data parameter must be a dictionary.")

    if "files" not in data or "folders" not in data or "extensions" not in data:
        raise TypeError(
            "The dictionary does not contain one of the following keys: 'files', 'folders', 'extensions'."
        )

    objects_in_directory = listdir(directory)

    def files_folders(key):

        # The "strictly" key will check whether the dictionary keys (files and folders) are tuples or lists.
        # If you use tuples, it means that the search must be rigorous, that is, if there is no such object,
        # the return will always be False.

        # If you use list in "files" or "folders", the search is tolerant, that is, if it finds a single object,
        # the return will always be True.

        strictly = False

        if data[key]:
            if type(data[key]) is tuple:
                strictly = True
            for item in data[key]:
                if strictly is False:
                    if item in objects_in_directory:
                        return True
                    return False
                else:
                    if item not in objects_in_directory:
                        return False
                    return True
        return False

    def finder_extensions():
        if data["extensions"]:
            lst = []
            for ext in data["extensions"]:
                for file in objects_in_directory:
                    if file.endswith(ext):
                        lst.append(join(directory, file))

            e = [splitext(f)[-1] for f in lst]
            for ext in data["extensions"]:
                if ext in e:
                    return True
        return False

    files = files_folders("files")
    folders = files_folders("folders")
    extensions = finder_extensions()

    if (files is False and folders is False) and extensions is False:
        return False

    return True



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['snakypy',
 'snakypy.zshpower',
 'snakypy.zshpower.commands',
 'snakypy.zshpower.commands.utils',
 'snakypy.zshpower.config',
 'snakypy.zshpower.database',
 'snakypy.zshpower.prompt',
 'snakypy.zshpower.prompt.sections',
 'snakypy.zshpower.prompt.sections.utils',
 'snakypy.zshpower.utils']

package_data = \
{'': ['*']}

install_requires = \
['docopt>=0.6.2,<0.7.0',
 'snakypy-helpers>=0.2.2,<0.3.0',
 'tomlkit>=0.7.1,<0.8.0']

entry_points = \
{'console_scripts': ['zshpower = snakypy.zshpower.cli:main',
                     'zshpower-draw = snakypy.zshpower.draw:main']}

setup_kwargs = {
    'name': 'zshpower',
    'version': '0.11.7',
    'description': 'ZSHPower is a theme for ZSH with a manager.',
    'long_description': '.. image:: https://raw.githubusercontent.com/snakypy/assets/master/zshpower/images/zshpower-transparent.png\n    :align: center\n    :alt: ZSHPower\n\n_________________\n\n.. image:: https://github.com/snakypy/zshpower/workflows/Tests/badge.svg\n    :target: https://github.com/snakypy/zshpower\n\n.. image:: https://img.shields.io/pypi/v/zshpower.svg\n    :target: https://pypi.python.org/pypi/zshpower\n    :alt: PyPI - ZSHPower\n\n.. image:: https://img.shields.io/pypi/wheel/zshpower\n    :target: https://pypi.org/project/wheel/\n    :alt: PyPI - Wheel\n\n.. image:: https://img.shields.io/pypi/pyversions/zshpower\n    :target: https://pyup.io/repos/github/snakypy/zshpower/\n    :alt: Python versions\n\n.. image:: https://img.shields.io/badge/code%20style-black-000000.svg\n    :target: https://github.com/psf/black\n    :alt: Black\n\n.. image:: https://img.shields.io/badge/%20imports-isort-%231674b1?style=flat&labelColor=ef8336\n    :target: https://pycqa.github.io/isort/\n    :alt: Isort\n\n.. image:: http://www.mypy-lang.org/static/mypy_badge.svg\n    :target: http://mypy-lang.org/\n    :alt: Mypy\n\n.. image:: https://pyup.io/repos/github/snakypy/zshpower/shield.svg\n   :target: https://pyup.io/repos/github/snakypy/zshpower/\n   :alt: Updates\n\n.. image:: https://img.shields.io/github/issues-raw/snakypy/zshpower\n    :target: https://github.com/snakypy/zshpower/issues\n    :alt: GitHub issues\n\n.. image:: https://img.shields.io/github/license/snakypy/zshpower\n    :target: https://github.com/snakypy/zshpower/blob/main/LICENSE\n    :alt: GitHub license\n\n_________________\n\n`ZSHPower` is a theme for ZSH; especially for the `Python`_ developer. Pleasant to look at, the **ZSHPower** comforts you with its colors and icons vibrant.\n\nInstalling **ZSHPower** is the easiest thing you will see in any existing theme for **ZSH**, because there is a manager.\n\nThe changes in the theme become more dynamic through a configuration file, where the user can make various combinations for the style of **ZSHPower**.\n\nThe **ZSHPower** supports installation along with `Oh My ZSH`_, where changes to: **enable** and **disable** an `Oh My ZSH`_ theme are easier, all in a simplified command line, without opening any files or creating symbolic links.\n\nIn addition, the **ZSHPower** manager downloads **Oh My Zsh** and the\n`zsh-autosuggestions`_ and `zsh-syntax-highlighting`_ plugins automatically, everything to make your ZSH very power.\n\n\nRequirements\n------------\n\nTo work correctly, you will first need:\n\n* `git`_ (v2.25 or recent) must be installed.\n* `zsh`_  (v5.2 or recent) must be installed.\n* `python`_ (v3.9 or recent) must be installed.\n* `sqlite3`_ (v3.35 or recent) must be installed.\n* `pip`_ (v21.0.1 or recent) must be installed.\n* `nerd fonts`_ must be installed.\n\n\nFeatures\n--------\n\n* `Oh My Zsh`_ installation automatically;*\n* Automatically install `zsh-autosuggestions`_ and `zsh-syntax-highlighting`_;\n* Automated installation and uninstallation;\n* Enable and disable `ZSHPower` anytime;\n* Upgrade `ZSHPower` effortlessly;\n* Reset the settings with one command only;\n* Personalized directory with truncate option;\n* Current Git branch and rich repo status:\n    *  — untracked changes;\n    *  — new files added;\n    *  — deleted files;\n    *  — new modified files;\n    *  — commits made;\n    *  — and more.\n* Application versions shown with `nerd fonts`_, they are:\n    * CMake, Crystal, Dart, Deno, Docker, Docker, Dotnet, Elixir, Erlang, Go, Gulp, Helm, Java, Julia, Kotlin, Nim, NodeJS, Ocaml, Perl, Php, Python, Ruby, Rust, Scala, Vagrant, Zig\n* Package versions such as Crystal, Helm, NodeJS, Python, Rust shown;\n* Shows the time in the upper right corner;\n* and, many other dynamic settings in `$HOME/.zshpower/config/zshpower.toml`.\n\n\\* features if used with **Oh My ZSH**.\n\n\nInstalling\n----------\n\n.. code-block:: shell\n\n    $ python3 -m pip install zshpower --user\n\nNOTE: It is recommended that you install for user rather than global.\n\nUsing\n-----\n\nRun the command below to set `ZSHPower`_ on your ZSH:\n\n.. code-block:: shell\n\n    $ zshpower init\n\nIf you want to use ZSHPower with `Oh My Zsh`_, use the **--omz** flag:\n\n.. code-block:: shell\n\n    $ zshpower init --omz\n\nFor more command information, run:\n\n.. code-block:: shell\n\n    $ zshpower --help\n\nMore information: https://github.com/snakypy/zshpower\n\nDonation\n--------\n\nClick on the image below to be redirected the donation forms:\n\n.. image:: https://raw.githubusercontent.com/snakypy/donations/main/svg/donate/donate-hand.svg\n    :width: 160 px\n    :height: 100px\n    :target: https://github.com/snakypy/donations/blob/main/README.md\n\n\nLicense\n-------\n\nThe gem is available as open source under the terms of the `MIT License`_ ©\n\nCredits\n-------\n\nSee, `AUTHORS`_.\n\nLinks\n-----\n\n* Code: https://github.com/snakypy/zshpower\n* Documentation: https://github.com/snakypy/zshpower/blob/main/README.md\n* Releases: https://pypi.org/project/zshpower/#history\n* Issue tracker: https://github.com/snakypy/zshpower/issues\n\n.. _AUTHORS: https://github.com/snakypy/zshpower/blob/main/AUTHORS.rst\n.. _Oh My Zsh: https://ohmyz.sh\n.. _zsh-autosuggestions: https://github.com/zsh-users/zsh-autosuggestions\n.. _zsh-syntax-highlighting: https://github.com/zsh-users/zsh-syntax-highlighting\n.. _ZSHPower: https://github.com/snakypy/zshpower\n.. _git: https://git-scm.com/downloads\n.. _zsh: http://www.zsh.org/\n.. _python: https://python.org\n.. _sqlite3: https://www.sqlite.org\n.. _pip: https://pip.pypa.io/en/stable/quickstart/\n.. _nerd fonts: https://www.nerdfonts.com/font-downloads\n.. _MIT License: https://github.com/snakypy/zshpower/blob/main/LICENSE\n.. _William Canin: http://williamcanin.github.io\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _`williamcanin/pypkg-cookiecutter`: https://github.com/williamcanin/pypkg-cookiecutter\n',
    'author': 'William C. Canin',
    'author_email': 'william.costa.canin@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/snakypy/zshpower',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from snakypy.zshpower.utils.catch import get_key


class JumpLine:
    def __init__(self, config: dict):
        self.enable: str = get_key(config, "general", "jump_line", "enable")

    def __str__(self):
        if self.enable:
            return "\n"
        return ""



from os import getcwd
from os.path import isfile, join
from subprocess import run

from snakypy.helpers.files import read_json

from snakypy.zshpower.config.base import Base
from snakypy.zshpower.prompt.sections.utils import (
    Color,
    Version,
    element_spacing,
    separator,
    symbol_ssh,
)
from snakypy.zshpower.utils.catch import get_key, verify_objects


class Gulp(Version, Base):
    def __init__(self, *args):
        super(Gulp, self).__init__()
        self.args: tuple = args
        self.finder = {
            "extensions": [],
            "folders": [],
            "files": ["gulpfile.js", "gulpfile.babel.js"],
        }
        self.key = "gulp"
        self.app_executable = "gulp"
        self.shorten = "gulp-"

    def get_version(self, space_elem: str = " ") -> str:
        # args[0]: dict = config file (toml)
        # args[1]: dict = database registers
        version_local = "node_modules/gulp/package.json"
        enable = get_key(self.args[0], self.key, "version", "enable")
        symbol = symbol_ssh(get_key(self.args[0], self.key, "symbol"), self.shorten)
        color = get_key(self.args[0], self.key, "color")
        prefix_color = get_key(self.args[0], self.key, "prefix", "color")
        prefix_text = element_spacing(get_key(self.args[0], self.key, "prefix", "text"))
        micro_version_enable = get_key(
            self.args[0], self.key, "version", "micro", "enable"
        )

        if enable and verify_objects(getcwd(), data=self.finder) is True:
            prefix = f"{Color(prefix_color)}{prefix_text}{Color().NONE}"
            if isfile(join(getcwd(), version_local)):
                parsed = read_json(join(getcwd(), version_local))
                if "version" in parsed:
                    version_ = parsed["version"]

                    if micro_version_enable:
                        version_format = f"{'{0[0]}.{0[1]}.{0[2]}'.format(version_.split('.'))}{space_elem}"
                    else:
                        version_format = (
                            f"{'{0[0]}.{0[1]}'.format(version_.split('.'))}{space_elem}"
                        )

                    return str(
                        (
                            f"{separator(self.args[0])}{prefix}"
                            f"{Color(color)}{symbol}"
                            f"Local {version_format}{Color().NONE}"
                        )
                    )
            else:
                return super().get(
                    self.args[0],
                    self.args[1],
                    self.key,
                    self.shorten,
                    space_elem=space_elem,
                )
        return ""

    def set_version(self, action: str = "") -> bool:
        command = run("gulp --version", capture_output=True, shell=True, text=True)
        version = command.stdout.split()[2]
        return super().set(command, version, self.app_executable, self.key, action)

    def __str__(self):
        return self.get_version()



from time import strftime

from snakypy.zshpower.prompt.sections.utils import Color, symbol_ssh
from snakypy.zshpower.utils.catch import get_key


class Timer:
    def __init__(self, config: dict):
        self.enable = get_key(config, "timer", "enable")
        self.symbol = symbol_ssh(get_key(config, "timer", "symbol"), "T:")
        self.color = (
            get_key(config, "timer", "color")
            if get_key(config, "general", "color", "enable") is True
            else "negative"
        )
        self.seconds_enable = get_key(config, "timer", "seconds", "enable")

    def __str__(self):
        if self.enable:
            get_timer = str(strftime("%H:%M"))
            if self.seconds_enable:
                get_timer = str(strftime("%H:%M:%S"))
            timer = f"{Color(self.color)}{self.symbol}" f"{get_timer}{Color().NONE}"
            return timer.strip()
        return ""



from os.path import join

from snakypy.helpers.catches import whoami
from snakypy.helpers.logging import Log
from snakypy.helpers.path import create as create_path

from snakypy.zshpower import __info__
from snakypy.zshpower.database.sql import sql


class Base:
    def __init__(self, home):
        self.HOME = home
        self.zshpower_home = join(self.HOME, f".{__info__['pkg_name']}")
        self.tbl_main = [item for item in sql().keys()][0]
        self.config_root = join(self.zshpower_home, "config")
        self.lib_root = join(self.zshpower_home, "lib")
        self.config_file = join(self.config_root, "zshpower.toml")
        self.database_root = join(self.zshpower_home, ".database")
        self.cache_root = join(self.zshpower_home, ".cache")
        self.logfile = join(self.zshpower_home, ".cache", "zshpower.log")
        self.database_path = join(self.database_root, "zshpower.sqlite3")
        # self.sync_path = f"/usr/local/bin/{__info__['pkg_name']}_sync.zsh"
        self.cron_d_path = "/etc/cron.d/"
        self.cron_path = join(self.cron_d_path, f"{__info__['pkg_name']}__{whoami()}")
        self.lib_main = join(self.lib_root, "main.lib")
        self.zsh_rc = join(self.HOME, ".zshrc")
        self.omz_root = join(self.HOME, ".oh-my-zsh")
        self.themes_folder = join(self.omz_root, "custom/themes")
        self.theme_symlink = join(
            self.themes_folder, f"{__info__['pkg_name']}.zsh-theme"
        )
        self.source_code = f".{__info__['pkg_name']}/lib/main.lib"
        self.plugins = ("zsh-syntax-highlighting", "zsh-autosuggestions")
        create_path(self.cache_root)
        self.log = Log(self.logfile)
